<?php
/**
 * A special member class for Ship-Simu
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 Ship-Simu Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ShipSimuMember extends ShipSimuBaseUser implements ManageableMember, BookableAccount {
	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className = __CLASS__) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Destructor for e.g. flushing pending updates to the database
	 *
	 * @return	void
	 */
	public function __destruct () {
		// Flush any updated entries to the database
		$this->flushPendingUpdates();

		// Call parent destructor
		parent::__destruct();
	}

	/**
	 * Creates an instance of this user class by a provided username. This
	 * factory method will check if username is already taken and if not
	 * so it will throw an exception.
	 *
	 * @param	$userName		Username we need a class instance for
	 * @return	$userInstance	An instance of this user class
	 * @throws	UsernameMissingException	If the username does not exist
	 */
	public final static function createMemberByUsername ($userName) {
		// Get a new instance
		$userInstance = new ShipSimuMember();

		// Set the username
		$userInstance->setUserName($userName);

		// Check if username exists
		if ($userInstance->ifUsernameExists() === false) {
			// Throw an exception here
			throw new UsernameMissingException(array($userInstance, $userName), self::EXCEPTION_USERNAME_NOT_FOUND);
		} // END - if

		// Return the instance
		return $userInstance;
	}

	/**
	 * Creates an instance of this user class by a provided email address. This
	 * factory method will not check if email address is there.
	 *
	 * @param	$email			Email address of the user
	 * @return	$userInstance	An instance of this user class
	 */
	public final static function createMemberByEmail ($email) {
		// Get a new instance
		$userInstance = new ShipSimuMember();

		// Set the username
		$userInstance->setEmail($email);

		// Return the instance
		return $userInstance;
	}

	/**
	 * Creates a user by a given request instance
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	$userInstance		An instance of this user class
	 * @todo	Add more ways of creating user instances
	 */
	public final static function createMemberByRequest (Requestable $requestInstance) {
		// Determine if by email or username
		if (!is_null($requestInstance->getRequestElement('username'))) {
			// Username supplied
			$userInstance = self::createMemberByUserName($requestInstance->getRequestElement('username'));
		} elseif (!is_null($requestInstance->getRequestElement('email'))) {
			// Email supplied
			$userInstance = self::createMemberByEmail($requestInstance->getRequestElement('email'));
		} else {
			// Unsupported mode
			$userInstance = new ShipSimuMember();
			$userInstance->partialStub("We need to add more ways of creating user classes here.");
			$userInstance->debugBackTrace();
			exit();
		}

		// Return the prepared instance
		return $userInstance;
	}

	/**
	 * Updates the last activity timestamp and last performed action in the
	 * database result. You should call flushPendingUpdates() to flush these updates
	 * to the database layer.
	 *
	 * @param	$requestInstance	A requestable class instance
	 * @return	void
	 */
	public function updateLastActivity (Requestable $requestInstance) {
		// Set last action
		$lastAction = $requestInstance->getRequestElement('action');

		// If there is no action use the default on
		if (is_null($lastAction)) {
			$lastAction = $this->getConfigInstance()->getConfigEntry('login_default_action');
		} // END - if

		// Get a critieria instance
		$searchInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

		// Add search criteria
		$searchInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUserName());
		$searchInstance->setLimit(1);

		// Now get another criteria
		$updateInstance = ObjectFactory::createObjectByConfiguredName('update_criteria_class');

		// And add our both entries
		$updateInstance->addCriteria("last_activity", date("Y-m-d H:i:s", time()));
		$updateInstance->addCriteria("last_action", $lastAction);

		// Add the search criteria for searching for the right entry
		$updateInstance->setSearchInstance($searchInstance);

		// Set wrapper class name
		$updateInstance->setWrapperConfigEntry('user_db_wrapper_class');

		// Remember the update in database result
		$this->getResultInstance()->add2UpdateQueue($updateInstance);
	}

	/**
	 * Books the given 'amount' in the request instance on the users "points
	 * account"
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	void
	 */
	public function bookAmountDirectly (Requestable $requestInstance) {
		// Get the points class from registry
		$pointsInstance = Registry::getRegistry()->getInstance('points');

		// Is the points instance null?
		if (is_null($pointsInstance)) {
			// Then get a new one
			$pointsInstance = ObjectFactory::createObjectByConfiguredName('user_points_class', array($this));

			// And store it in registry
			Registry::getRegistry()->addInstance('points', $pointsInstance);
		} // END - if

		// Get the amount
		$amount = $requestInstance->getRequestElement('amount');

		// Call the method for booking points
		$pointsInstance->bookPointsDirectly($amount);
	}

	/**
	 * Flushs all pending updates to the database layer
	 *
	 * @return	void
	 */
	public function flushPendingUpdates () {
		// Is the object valid?
		if (!$this->getResultInstance() instanceof SearchableResult) {
			// Abort here
			return;
		} // END - if

		// Do we have data to update?
		if ($this->getResultInstance()->ifDataNeedsFlush()) {
			// Get a database wrapper
			$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

			// Yes, then send the whole result to the database layer
			$wrapperInstance->doUpdateByResult($this->getResultInstance());
		} // END - if
	}
}

// [EOF]
?>
